"use strict";
// Copyright (c) Nicholas Earl
// Distributed under the terms of the Modified BSD License.
Object.defineProperty(exports, "__esModule", { value: true });
const base_1 = require("@jupyter-widgets/base");
const version_1 = require("./version");
const base_2 = require("@jupyter-widgets/base");
const messaging_1 = require("@phosphor/messaging");
const widgets_1 = require("@phosphor/widgets");
// import * as GoldenLayout from 'golden-layout';
const GoldenLayout = require('golden-layout');
require("golden-layout/src/css/goldenlayout-base.css");
require("golden-layout/src/css/goldenlayout-light-theme.css");
require("./custom.css");
class IPyGLModel extends base_1.DOMWidgetModel {
    defaults() {
        return Object.assign({}, super.defaults(), { _model_name: IPyGLModel.model_name, _model_module: IPyGLModel.model_module, _model_module_version: IPyGLModel.model_module_version, _view_name: IPyGLModel.view_name, _view_module: IPyGLModel.view_module, _view_module_version: IPyGLModel.view_module_version, children: [] });
    }
}
IPyGLModel.serializers = Object.assign({}, base_1.DOMWidgetModel.serializers, { children: { deserialize: base_2.unpack_models } });
IPyGLModel.model_name = 'IPyGLModel';
IPyGLModel.model_module = version_1.MODULE_NAME;
IPyGLModel.model_module_version = version_1.MODULE_VERSION;
IPyGLModel.view_name = 'IPyGLView'; // Set to null if no view
IPyGLModel.view_module = version_1.MODULE_NAME; // Set to null if no view
IPyGLModel.view_module_version = version_1.MODULE_VERSION;
exports.IPyGLModel = IPyGLModel;
class IPyGLView extends base_1.DOMWidgetView {
    constructor(...args) {
        super(...args);
        this.children_views = null;
        this.children_views = new base_1.ViewList(this.add_child_model, v => null, this);
        this.listenTo(this.model, 'change:children', this.update_children);
        this.mountPoint = document.createElement('div');
        this.mountPoint.setAttribute('id', 'ipygl-area');
        let config = {
            settings: {
                selectionEnabled: this.model.get('selection_enabled')
            },
            dimensions: {
                borderWidth: 5,
                minItemHeight: 10,
                minItemWidth: 10,
                headerHeight: 20,
                dragProxyWidth: 300,
                dragProxyHeight: 200
            },
            content: []
        };
        this.layout = new GoldenLayout(config, this.mountPoint);
        this.layout._isFullPage = true;
        this.layout.registerComponent('gl-tab-content', function (container, state) {
            if (state.widget) {
                // Although the docs say that the event is emitted after the container is
                // added to the DOM, this doesn't seem to be the case. Instead, hold the
                // widget as the result of a setTimeout callback.
                container.on('open', () => {
                    setTimeout(() => {
                        widgets_1.Widget.attach(state.widget, container.getElement()[0]);
                    }, 0);
                });
            }
            else {
                container.getElement().html(state.content);
            }
        });
    }
    render() {
        this.el.appendChild(this.mountPoint);
        this.layout.init();
    }
    processPhosphorMessage(msg) {
        super.processPhosphorMessage(msg);
        switch (msg.type) {
            case 'resize':
                this.layout.updateSize();
                break;
            case 'after-show':
                this.layout.updateSize();
                break;
            default:
                break;
        }
    }
    update_children() {
        if (this.children_views == null)
            return;
        this.children_views.update(this.model.get('children')).then((views) => {
            // Notify all children that their sizes may have changed.
            views.forEach((view) => {
                messaging_1.MessageLoop.postMessage(view.pWidget, widgets_1.Widget.ResizeMessage.UnknownSize);
            });
        });
    }
    add_child_model(model, index) {
        return this.create_child_view(model).then((view) => {
            let parent = this.layout.root;
            if (this.layout.root.contentItems.length > 0) {
                parent = this.layout.root.contentItems[0];
            }
            parent.addChild({
                type: 'component',
                componentName: 'gl-tab-content',
                componentState: { widget: view.pWidget },
                isClosable: true
            });
            return view;
        }).catch(reject('Could not add child view to box', true));
    }
    remove() {
        this.children_views = null;
        super.remove();
    }
}
exports.IPyGLView = IPyGLView;
function reject(message, log) {
    return function promiseRejection(error) {
        let wrapped_error = new base_2.WrappedError(message, error);
        if (log) {
            console.error(wrapped_error);
        }
        return Promise.reject(wrapped_error);
    };
}
//# sourceMappingURL=widget.js.map